/*
 * Library to access the Windows SuperFetch database (AGDB) file format
 *
 * Copyright (C) 2014-2018, Joachim Metz <joachim.metz@gmail.com>
 *
 * Refer to AUTHORS for acknowledgements.
 *
 * This software is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this software.  If not, see <http://www.gnu.org/licenses/>.
 */

#if !defined( _LIBAGDB_H )
#define _LIBAGDB_H

#include <libagdb/codepage.h>
#include <libagdb/definitions.h>
#include <libagdb/error.h>
#include <libagdb/extern.h>
#include <libagdb/features.h>
#include <libagdb/types.h>

#include <stdio.h>

#if defined( LIBAGDB_HAVE_BFIO )
#include <libbfio.h>
#endif

#if defined( __cplusplus )
extern "C" {
#endif

/* -------------------------------------------------------------------------
 * Support functions
 * ------------------------------------------------------------------------- */

/* Returns the library version
 */
LIBAGDB_EXTERN \
const char *libagdb_get_version(
             void );

/* Returns the access flags for reading
 */
LIBAGDB_EXTERN \
int libagdb_get_access_flags_read(
     void );

/* Retrieves the narrow system string codepage
 * A value of 0 represents no codepage, UTF-8 encoding is used instead
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_get_codepage(
     int *codepage,
     libagdb_error_t **error );

/* Sets the narrow system string codepage
 * A value of 0 represents no codepage, UTF-8 encoding is used instead
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_set_codepage(
     int codepage,
     libagdb_error_t **error );

/* Determines if a file contains an AGDB file signature
 * Returns 1 if true, 0 if not or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_check_file_signature(
     const char *filename,
     libagdb_error_t **error );

#if defined( LIBAGDB_HAVE_WIDE_CHARACTER_TYPE )

/* Determines if a file contains an AGDB file signature
 * Returns 1 if true, 0 if not or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_check_file_signature_wide(
     const wchar_t *filename,
     libagdb_error_t **error );

#endif /* defined( LIBAGDB_HAVE_WIDE_CHARACTER_TYPE ) */

#if defined( LIBAGDB_HAVE_BFIO )

/* Determines if a file contains an AGDB file signature using a Basic File IO (bfio) handle
 * Returns 1 if true, 0 if not or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_check_file_signature_file_io_handle(
     libbfio_handle_t *file_io_handle,
     libagdb_error_t **error );

#endif /* defined( LIBAGDB_HAVE_BFIO ) */

/* -------------------------------------------------------------------------
 * Notify functions
 * ------------------------------------------------------------------------- */

/* Sets the verbose notification
 */
LIBAGDB_EXTERN \
void libagdb_notify_set_verbose(
      int verbose );

/* Sets the notification stream
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_notify_set_stream(
     FILE *stream,
     libagdb_error_t **error );

/* Opens the notification stream using a filename
 * The stream is opened in append mode
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_notify_stream_open(
     const char *filename,
     libagdb_error_t **error );

/* Closes the notification stream if opened using a filename
 * Returns 0 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_notify_stream_close(
     libagdb_error_t **error );

/* -------------------------------------------------------------------------
 * Error functions
 * ------------------------------------------------------------------------- */

/* Frees an error
 */
LIBAGDB_EXTERN \
void libagdb_error_free(
      libagdb_error_t **error );

/* Prints a descriptive string of the error to the stream
 * Returns the number of printed characters if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_error_fprint(
     libagdb_error_t *error,
     FILE *stream );

/* Prints a descriptive string of the error to the string
 * The end-of-string character is not included in the return value
 * Returns the number of printed characters if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_error_sprint(
     libagdb_error_t *error,
     char *string,
     size_t size );

/* Prints a backtrace of the error to the stream
 * Returns the number of printed characters if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_error_backtrace_fprint(
     libagdb_error_t *error,
     FILE *stream );

/* Prints a backtrace of the error to the string
 * The end-of-string character is not included in the return value
 * Returns the number of printed characters if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_error_backtrace_sprint(
     libagdb_error_t *error,
     char *string,
     size_t size );

/* -------------------------------------------------------------------------
 * File functions
 * ------------------------------------------------------------------------- */

/* Creates a file
 * Make sure the value file is referencing, is set to NULL
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_initialize(
     libagdb_file_t **file,
     libagdb_error_t **error );

/* Frees a file
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_free(
     libagdb_file_t **file,
     libagdb_error_t **error );

/* Signals the libagdb file to abort its current activity
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_signal_abort(
     libagdb_file_t *file,
     libagdb_error_t **error );

/* Opens a file
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_open(
     libagdb_file_t *file,
     const char *filename,
     int access_flags,
     libagdb_error_t **error );

#if defined( LIBAGDB_HAVE_WIDE_CHARACTER_TYPE )

/* Opens a file
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_open_wide(
     libagdb_file_t *file,
     const wchar_t *filename,
     int access_flags,
     libagdb_error_t **error );

#endif /* defined( LIBAGDB_HAVE_WIDE_CHARACTER_TYPE ) */

#if defined( LIBAGDB_HAVE_BFIO )

/* Opens a file using a Basic File IO (bfio) handle
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_open_file_io_handle(
     libagdb_file_t *file,
     libbfio_handle_t *file_io_handle,
     int access_flags,
     libagdb_error_t **error );

#endif /* defined( LIBAGDB_HAVE_BFIO ) */

/* Closes a file
 * Returns 0 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_close(
     libagdb_file_t *file,
     libagdb_error_t **error );

/* Retrieves the number of volumes
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_get_number_of_volumes(
     libagdb_file_t *file,
     int *number_of_volumes,
     libagdb_error_t **error );

/* Retrieves a specific volume information
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_get_volume_information(
     libagdb_file_t *file,
     int volume_index,
     libagdb_volume_information_t **volume_information,
     libagdb_error_t **error );

/* Retrieves the number of sources
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_get_number_of_sources(
     libagdb_file_t *file,
     int *number_of_sources,
     libagdb_error_t **error );

/* Retrieves a specific source information
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_get_source_information(
     libagdb_file_t *file,
     int source_index,
     libagdb_source_information_t **source_information,
     libagdb_error_t **error );

/* -------------------------------------------------------------------------
 * Volume information functions
 * ------------------------------------------------------------------------- */

/* Frees volume information
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_free(
     libagdb_volume_information_t **volume_information,
     libagdb_error_t **error );

/* Retrieves the 64-bit filetime value containing the volume creation date and time
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_get_creation_time(
     libagdb_volume_information_t *volume_information,
     uint64_t *creation_time,
     libagdb_error_t **error );

/* Retrieves the serial number
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_get_serial_number(
     libagdb_volume_information_t *volume_information,
     uint32_t *serial_number,
     libagdb_error_t **error );

/* Retrieves the size of the UTF-8 encoded device path
 * The returned size includes the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_get_utf8_device_path_size(
     libagdb_volume_information_t *volume_information,
     size_t *utf8_string_size,
     libagdb_error_t **error );

/* Retrieves the UTF-8 encoded device path
 * The size should include the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_get_utf8_device_path(
     libagdb_volume_information_t *volume_information,
     uint8_t *utf8_string,
     size_t utf8_string_size,
     libagdb_error_t **error );

/* Retrieves the size of the UTF-16 encoded device path
 * The returned size includes the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_get_utf16_device_path_size(
     libagdb_volume_information_t *volume_information,
     size_t *utf16_string_size,
     libagdb_error_t **error );

/* Retrieves the UTF-16 encoded device path
 * The size should include the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_get_utf16_device_path(
     libagdb_volume_information_t *volume_information,
     uint16_t *utf16_string,
     size_t utf16_string_size,
     libagdb_error_t **error );

/* Retrieves the number of files
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_get_number_of_files(
     libagdb_volume_information_t *volume_information,
     int *number_of_files,
     libagdb_error_t **error );

/* Retrieves a specific file information
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_volume_information_get_file_information(
     libagdb_volume_information_t *volume_information,
     int file_index,
     libagdb_file_information_t **file_information,
     libagdb_error_t **error );

/* -------------------------------------------------------------------------
 * File information functions
 * ------------------------------------------------------------------------- */

/* Frees file information
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_information_free(
     libagdb_file_information_t **file_information,
     libagdb_error_t **error );

/* Retrieves the size of the UTF-8 encoded path
 * The returned size includes the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_information_get_utf8_path_size(
     libagdb_file_information_t *file_information,
     size_t *utf8_string_size,
     libagdb_error_t **error );

/* Retrieves the UTF-8 encoded path
 * The size should include the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_information_get_utf8_path(
     libagdb_file_information_t *file_information,
     uint8_t *utf8_string,
     size_t utf8_string_size,
     libagdb_error_t **error );

/* Retrieves the size of the UTF-16 encoded path
 * The returned size includes the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_information_get_utf16_path_size(
     libagdb_file_information_t *file_information,
     size_t *utf16_string_size,
     libagdb_error_t **error );

/* Retrieves the UTF-16 encoded path
 * The size should include the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_file_information_get_utf16_path(
     libagdb_file_information_t *file_information,
     uint16_t *utf16_string,
     size_t utf16_string_size,
     libagdb_error_t **error );

/* -------------------------------------------------------------------------
 * Source information functions
 * ------------------------------------------------------------------------- */

/* Frees source information
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_source_information_free(
     libagdb_source_information_t **source_information,
     libagdb_error_t **error );

/* Retrieves the size of the UTF-8 encoded executable filename
 * The returned size includes the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_source_information_get_utf8_executable_filename_size(
     libagdb_source_information_t *source_information,
     size_t *utf8_string_size,
     libagdb_error_t **error );

/* Retrieves the UTF-8 encoded executable filename
 * The size should include the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_source_information_get_utf8_executable_filename(
     libagdb_source_information_t *source_information,
     uint8_t *utf8_string,
     size_t utf8_string_size,
     libagdb_error_t **error );

/* Retrieves the size of the UTF-16 encoded executable filename
 * The returned size includes the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_source_information_get_utf16_executable_filename_size(
     libagdb_source_information_t *source_information,
     size_t *utf16_string_size,
     libagdb_error_t **error );

/* Retrieves the UTF-16 encoded executable filename
 * The size should include the end of string character
 * Returns 1 if successful or -1 on error
 */
LIBAGDB_EXTERN \
int libagdb_source_information_get_utf16_executable_filename(
     libagdb_source_information_t *source_information,
     uint16_t *utf16_string,
     size_t utf16_string_size,
     libagdb_error_t **error );

#if defined( __cplusplus )
}
#endif

#endif /* !defined( _LIBAGDB_H ) */

